import .LinearAlgebra ;
import .LeastSquares ;
import .Random ;

array Higuchi( array serie, int kmin , int kmax, float error_percent )
{
  float suma = 0.0 ;
  int n = sizeof(serie);
  int num_k = kmax - kmin + 1 ;
  array llista_k = (array) NMatrix( num_k, 1 );
  array llista_lk = (array) NMatrix( num_k, 1 ) ;
  array llista_errors = (array) NMatrix( num_k, 1 ) ;
  float sumak = 0.0 ;
  int index = -1 ;
  for ( int k = kmin ; k <= kmax ; ++k )
    {
      sumak = 0.0 ;
      ++index ;
      for ( int m = 0 ; m < k ; ++m )
	{
	  int i_max = ( n - m - 1 ) / k ;
	  for ( int i = 0 ; i < i_max ; ++i )
	    {
	      float element_1 = serie[ (m + (i + 1) * k ) ] ;
	      float element_2 = serie[ (m + i*k) ] ;
	      suma += abs( element_1 - element_2 );
	    }
	  suma = suma * ( (float) ( n - 1 ) / ( i_max * k * k * k ) ) ;
	  sumak += suma ;
	}
      llista_k[index][0] = log((float)k) ;
      llista_lk[index][0] = log(sumak);
      if (log(sumak) == 0 )
	{
	  llista_errors[index][0] =  0.1 ;
	}
      else
	{
	  llista_errors[index][0] = log(sumak) * error_percent ;
	}
    }
  array resultat = LinearLeastSquaresStraightLine ( NMatrix (llista_k) , NMatrix( llista_lk) , NMatrix (llista_errors) ) ;
  return ({ - resultat[0] , resultat[1] , resultat[2] , resultat[3] }) ;
}

array GenerateLinearData( float a, float b, int n ) 
{
  array serie = ( array ) NMatrix( n , 1 ) ;
  for ( int i = 0 ; i < n ; ++i )
    {
      serie[i] = a * i + b ;
    }
  return serie ;
}

array GenerateSinusoidalData (int n ) 
{
  array serie = ( array ) NMatrix(n, 1 ) ;
  for ( int i = 0 ; i < n ; ++i )
    {
      serie[i] = cos ( 2 * Math.pi * i / n ) ;
    }
  return serie ;
}

array GenerateLogisticData( float r, float x0, int n , int transitori) 
{
  array serie = ( array ) NMatrix( n , 1 ) ;
  float valor = x0 ;
  if ( ! ( r < 4.0 && r > 0.0 ) )
    {
      throw(({"Invalid r value\n", backtrace()}));
    }
  for ( int i = 0 ; i < transitori ; ++i )
    {
       valor = valor * r * ( 1 - valor ) ;
    }
  for ( int i = 0 ; i < n ; ++i )
    {
      valor = valor * r * ( 1 - valor ) ;
      serie[i] = valor ;
      write("Valor "  + i + " = " + valor + " \n");
    }
  return serie ;
}
  
array GenerateHiguchiData ( int n ) 
{
  int n_aleatoris = n + 1000 , limit_superior = 0 ;;
  array serie = ( array ) NMatrix(n,1) ;
  array aleatoris = ( array ) NMatrix ( n + 1000 , 1 ) ;
  float suma = 0.0 ;
  RandomSelfInit();
  for(int i = 0 ; i < n_aleatoris ; ++i )
    {
      aleatoris[i] = GaussDev( 0.0 , 1.0 ) ;
    }
  for( int i = 0 ; i < n ; ++i )
    {
      suma = 0.0 ;
      limit_superior = i + 1000 ;
      for ( int k = 0; k < limit_superior ; ++k )
	{
	  suma += aleatoris[k] ;
	}
      serie[i] = suma ;
    }
  return serie ;
}

array BurlagaKlein ( array serie, int kmin , int kmax, float error_percent ) 
{
	float suma1 = 0.0, suma2 = 0.0 ;
	int n = sizeof(serie);
	int num_k = kmax - kmin + 1 ;
	array llista_k = (array) NMatrix( num_k , 1 );
	array llista_lk = (array) NMatrix ( num_k, 1 );
	array llista_errors = (array) NMatrix( num_k, 1 );
	float sumak = 0.0 ;
	int index = -1 ;
	int nmax = 0 ;
	float terme1, terme2;
	for(int k = kmin ; k <= kmax ; ++k )
	{
		nmax = (n - k)/k ;
		sumak = 0.0 ;
		++index ;
		for(int i = 0 ; i < nmax ; ++i )
		{
			suma1 = 0.0 ;
			suma2 = 0.0 ;
			for(int m = 0 ; m < k ; ++m )
			{
				terme1 = serie[m + i*k] ;
				terme2 = serie[m + (i + 1 )*k];
				suma1 += terme1 ;
				suma2 += terme2 ;
			}	
			sumak += abs( suma1 - suma2 );
		}
		sumak = sumak / ((float) k*k) ;
		llista_k[index][0] = log( (float) k);
		llista_lk[index][0] = log( sumak );
		if ( log(sumak) == 0.0 )
		{
			llista_errors[index][0] = 0.1 ;
		}
		else 
		{
			llista_errors[index][0] = log(sumak) * error_percent ;
		}
	}
	array resultat = LinearLeastSquaresStraightLine(NMatrix(llista_k) , NMatrix(llista_lk) , NMatrix(llista_errors ));
	return ({ -resultat[0] , resultat[1] , resultat[2], resultat[3] }) ;
}


array SimpleFractalDimension( array serie, int kmin , int kmax, float error_percent )
{
  float suma = 0.0 ;
  int n = sizeof(serie);
  int num_k = kmax - kmin + 1 ;
  array llista_k = (array) NMatrix( num_k, 1 );
  array llista_lk = (array) NMatrix( num_k, 1 ) ;
  array llista_errors = (array) NMatrix( num_k, 1 ) ;
  int index = -1 ;
  for ( int k = kmin ; k <= kmax ; ++k )
    {
      suma = 0.0 ;
      ++index ;
      int i_max = ( n  - 1 ) / k ;
      for ( int i = 0 ; i < i_max ; ++i )
	{
	  float element_1 = serie[  (i + 1) * k  ] ;
	  float element_2 = serie[  i*k ] ;
	  suma += abs( element_1 - element_2 );
	}
      suma = suma * ( (float) ( n - 1 ) / ( i_max * k * k  ) ) ;
      llista_k[index][0] = log((float)k) ;
      llista_lk[index][0] = log(suma);
      if (log(suma) == 0 )
	{
	  llista_errors[index][0] =  0.1 ;
	}
      else
	{
	  llista_errors[index][0] = log(suma) * error_percent ;
	}
    }
  array resultat = LinearLeastSquaresStraightLine ( NMatrix (llista_k) , NMatrix( llista_lk) , NMatrix (llista_errors) ) ;
  return ({ - resultat[0] , resultat[1] , resultat[2] , resultat[3] }) ;
}


int main ()
{
  array serie = GenerateLinearData ( 1.0 , 2.5 , 100 ) ;
  array resultat =   Higuchi( serie, 1,20, 0.1 ) ;
  write("La dimensió fractal del test nº 1 ha resultat ser " +  resultat[0] + "\n" );
  resultat = BurlagaKlein( serie, 1 , 20 , 0.1 ) ;
  write("El mateix càlcul fet amb el mètode de Burlaga i Klein dona com a resultat " +  resultat[0] + "\n" );
  resultat = SimpleFractalDimension( serie, 1 , 20 , 0.1) ;
  write("El mateix càlcul amb el mètode simple dona " + resultat[0] + "\n");
  serie = GenerateSinusoidalData( 100 ) ;
  resultat = Higuchi( serie, 1, 20 , 0.1 ) ;
  write("La dimensió fractal del test nº 2 ha resultat ser " + resultat[0] + "\n" );
  resultat = BurlagaKlein( serie, 1 , 20 , 0.1 ) ;
  write("El mateix càlcul fet amb el mètode de Burlaga i Klein dona com a resultat " +  resultat[0] + "\n" );
  resultat = SimpleFractalDimension( serie , 1 , 20 , 0.1) ;
  write("El mateix càlcul amb el mètode simple dona " + resultat[0] + "\n");
  write("Generant dades per al test d'Higuchi...\n");
  serie = GenerateHiguchiData( 10000 ) ;
  write("Ajustant les dades generades... \n");
  resultat = Higuchi( serie, 1, 50, 0.1 ) ;
  write("La dimensió fractal del test nº 3 ha resultat ser " + resultat[0] + "\n" );
  resultat = BurlagaKlein( serie, 1 , 50 , 0.1 ) ;
  write("El mateix càlcul fet amb el mètode de Burlaga i Klein dona com a resultat " +  resultat[0] + "\n" );
  resultat = SimpleFractalDimension( serie, 1 , 50 , 0.1) ;
  write("El mateix càlcul amb el mètode simple " + resultat[0] + "\n");
  write("Generant serie de dades extensiva\n");
  serie = GenerateLinearData( 2.5, 7.8 ,  50000 ) ;
  write("Calculant dimensió fractal\n");
  resultat = Higuchi ( serie , 1 , 50 , 0.1 ) ;
  write("El resultat del test extensiu en memòria ha sigut " + resultat[0] + "\n" );
  resultat = BurlagaKlein( serie, 1 , 50 , 0.1 ) ;
  write("El mateix càlcul fet amb el mètode de Burlaga i Klein dona com a resultat " +  resultat[0] + "\n" );
  resultat = SimpleFractalDimension( serie , 1 , 50 , 0.1 );
  write("El mateix amb el mètode simple dona " + resultat[0] + "\n");
  //  serie = GenerateLogisticData( 3.74 , 0.2 , 500, 1000 );
  //  resultat = Higuchi( serie , 1 , 50 , 0.1 );
  //  write("El resultat de les dades logístiques ha sigut " + resultat[0] + "\n" );
  
  return 0;
}


